<?php

namespace Mcnd\CLI;

use Laminas\Diactoros\Response\TextResponse;
use Notes\CLI\CommandFetcher;

use Psr\Http\Message\ResponseFactoryInterface,
    Psr\Container\ContainerInterface,
    Psr\Http\Message\ResponseInterface,
    Psr\Http\Message\ServerRequestInterface,
    Psr\Http\Server\MiddlewareInterface,
    Psr\Http\Server\RequestHandlerInterface;

class CliMiddleware implements MiddlewareInterface
{
    public \Closure $defaultResponse;
    
    public ContainerInterface $container;

    public null|CommandFetcher $fetcher;

    public CommandStack $commands;

    protected array $receivedCommandArgv;

    protected array $parsedCommandArgv;

    protected array $parsedOptions = [];

    public function __construct(ContainerInterface $container, callable $defaultResponseInterface = null, null|CommandFetcher $fetcher = null, null|CommandStack $stack = null) {
        $this->defaultResponse = $defaultResponseInterface;
        $this->container = $container;
        $this->fetcher = $fetcher;
        $this->commands = $stack ?? new CommandStack();
    }

    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler) : ResponseInterface
    {
        if (php_sapi_name() === 'cli' OR defined('STDIN')) {
            if ($this->fetcher) {
                $this->getCommandsFromAttributes();
            }

            $cliRequest = new CliRequest($request);

            # $cliRequest->setOptions($commands->getOptions());

            if ( $command = $this->commands->matchCommand($cliRequest->command, $cliRequest->options) ) {
                if ($command->callback) {
                    if (is_string($command->callback)) {
                        list($class, $method) = explode('::', $command->callback);

                        return $this->container->make($class)->{$method}($request->withAttribute('cli.command', $command)->withAttribute('cli.request', $cliRequest), []);
                    }
                }
                else {
                    return $this->printCommand($command);
                }
            }


            return call_user_func($this->defaultResponse);
        }

        return $handler->handle($request);
    }

    public function getCommandsFromAttributes() : self
    {
        $this->commands->merge($this->fetcher->compile());

        return $this;
    }

    private function printCommand(Command $command) : ResponseInterface
    {
        $text = [
            $command->description, "",
            "Usage:",
            "  command [options] [arguments]", "",
            "Options:",
            "  OPTIONS HERE", "",
            "Available commands:",
            "  COMMANDS HERE", "",
        ];

        return new TextResponse( implode(PHP_EOL, $text) );
    }
}

